from conf.configuration import *
from topic_modeling.topic_modeling import *
import logging




def convert_topic_vectors_from_cluster(ontology,model):
    corpora = load_corpora_list()
    logging.warning("preprocessing vectors of %d corpus modeled with %s and %s"%(len(corpora)),ontology,model)
    for corpus in corpora:
        logging.warning("preprocessing %s "%corpus)
        read_cluster_topic_vectors(corpus,ontology,model)


convert_topic_vectors_from_cluster('debatepedia','word2vec-esa-10')


from conf.configuration import *
import pandas as pd
from utils.parts import *
import random
import csv
import numpy as np
from topic_modeling.topics import *
def produce_top_k_suggestions(dataset,ontology,model,k,debug=False):
    path_top_k_topic_path = get_path_top_k_topics_per_document(dataset,ontology,model,k)
    topics,topic_ids = load_topics_with_ids('ontology-'+ontology)
    top_k_document_dict={}
    top_k_document_dict['id']=[]
    top_k_document_dict['topic-id']=[]

    for document_vectors_ids in parts_vectors(dataset,ontology,model):
        for document_vector_id in document_vectors_ids:
            document_vector = document_vector_id[0].copy()
            document_id=document_vector_id[1]
            for i in range(k):
                top_topic_index = np.argmax(document_vector)
                top_topic= topics[top_topic_index]
                top_topic_weight= document_vector[top_topic_index]
                top_topic_id=topic_ids[top_topic_index]
                document_vector[top_topic_index]=0
                top_k_document_dict['id'].append(document_id)
                top_k_document_dict['topic-id'].append(top_topic_id)

    df_top_k_topic=pd.DataFrame(top_k_document_dict)
    log_size("top-k-topics",df_top_k_topic.shape[0])
    df_top_k_topic.to_csv(path_top_k_topic_path,quotechar='"',sep=",",quoting=csv.QUOTE_ALL,encoding="utf-8",index=False)







import logging
import pandas as pd
from conf.configuration import *
import csv
import argument_esa_model.esa_all_terms
from topic_modeling import *
def load_documents(corpus):
    granularity = get_granularity(corpus)
    if granularity =='document':
        dataset_preprocessed_path =get_path_preprocessed_documents(corpus)
        df_documents= pd.read_csv(dataset_preprocessed_path,quotechar='"',sep="|",quoting=csv.QUOTE_ALL,encoding="utf-8")
        ids = list(df_documents['document-id'])
        texts= list(df_documents['document'])
    else:
        dataset_preprocessed_path =get_path_preprocessed_arguments(corpus)
        df_arguments= pd.read_csv(dataset_preprocessed_path,quotechar='"',sep="|",quoting=csv.QUOTE_ALL,encoding="utf-8")
        ids = list(df_arguments['argument-id'])
        texts= list(df_arguments['argument'])

    return texts,ids

def model_text(topic_ontology,text):
    path_topic_model = get_path_topic_model('ontology-'+topic_ontology,'esa')
    path_word2vec_model = get_path_topic_model('word2vec','word2vec')
    path_word2vec_vocab = get_path_vocab('word2vec')

    topic_dict = argument_esa_model.esa_all_terms.model_topic(path_topic_model,path_word2vec_model,path_word2vec_vocab,'cos',text)
    sorted_topic_dict = sorted(topic_dict )
    return  sorted_topic_dict.keys()


def save_argument_vectors(argument_ids,argument_vectors,path):
    columns = {}
    columns['argument-id']=argument_ids
    columns['document-vector']:argument_vectors
    df_argument_vectors = pd.DataFrame(columns)
    df_argument_vectors.to_pickle(path)
def save_document_vectors(document_ids,document_vectors,path):
    columns = {}
    columns['document-id']=document_ids
    columns['document-vector']=document_vectors

    df_document_vectors = pd.DataFrame(columns)
    df_document_vectors.to_pickle(path)


def configure_logging(corpus):
    file_name="../logs/model-%s.log"%corpus
    logging.basicConfig(filename=file_name,level=logging.DEBUG)
def mode_corpus(corpus):
    configure_logging(corpus)

    texts,ids = load_documents(corpus)

    granularity = get_granularity(corpus)
    logging.warning("Read %d %s"%(len(texts),granularity))
    topic_ontologies = get_topic_ontologies()

    for topic_ontology in topic_ontologies:
        logging.warning("Modeling corpus %s with ontology %s "%(corpus,topic_ontology))
        argument_esa_model.esa_all_terms.initialize_model()
        vectors=esa_model(topic_ontology,texts)

        if granularity=='document':
            path = get_path_document_vectors(corpus,topic_ontology,'esa')
            save_document_vectors(ids,vectors,path)
        else:
            path = get_path_argument_vectors(corpus,topic_ontology,'esa')
            save_argument_vectors(ids,vectors,path)
mode_corpus('aifdb')


import matplotlib.pyplot as plt
from conf.configuration import *
import matplotlib

import numpy as np
import pandas as pd

def load_topic_ontology_map(ontology):
    path_ontology_topics=get_path_topics('ontology-'+ontology)
    df_ontology_topics=pd.read_csv(path_ontology_topics,sep=",",encoding="utf-8",dtype={'id':str})
    topic_map={}
    for i,ontology_topic_record in df_ontology_topics.iterrows():
        topic_map[ontology_topic_record['id']]=ontology_topic_record['name']
    return topic_map

def produce_corpora_count_over_topic(ontology):
    topic_map=load_topic_ontology_map(ontology)
    path_corpora_over_topic_count_ontology=get_path_histogram_two_parameters('ontology-'+ontology,'corpora-count','ontology-topic')
    path_figure_corpora_over_topic_ontology_count=get_path_figure_histogram_two_parameters('ontology-'+ontology,'corpora-count','ontology-topic')
    path_ground_truth_corpora=get_path_ground_truth_corpora('topic-inventory',ontology)
    df_ground_truth_corpora=pd.read_csv(path_ground_truth_corpora,sep=",",encoding="utf-8",dtype={'ontology-topic-id':str})
    df_ground_truth_corpora=df_ground_truth_corpora[~df_ground_truth_corpora['ontology-topic-id'].isin(['-1','-2'])]
    df_ground_truth_corpora['ontology-topic']=df_ground_truth_corpora['ontology-topic-id'].apply(lambda topic_id:topic_map[topic_id] )
    corpora_count_over_topic=df_ground_truth_corpora.groupby('ontology-topic').agg({'corpus':'count','ontology-topic-id':'unique'
                                                                                    }).sort_values('corpus')
    corpora_count_over_topic.to_csv(path_corpora_over_topic_count_ontology,sep=",",encoding="utf-8")


def shorten_labels(top_k_ontology_topics,max_label_size):
    ticklabels=[]
    for i,ticklabel in enumerate(top_k_ontology_topics):
        if len(ticklabel) > max_label_size:
            ticklabels.append(ticklabel[:max_label_size])
        else:
            to_add=max_label_size-len(ticklabel)
            ticklabel=ticklabel+to_add*" "

            ticklabels.append(ticklabel)
    return ticklabels



def prdoce_corpora_count_over_topic_figure(ontology, k, radial_step,max_label_size,size,offset):
    print(ontology)
    matplotlib.rcParams['pdf.fonttype']='truetype'
    path_corpora_over_topic_count_ontology=get_path_histogram_two_parameters('ontology-'+ontology,'corpora-count','ontology-topic')
    path_figure_corpora_over_topic_ontology_count=get_path_figure_histogram_two_parameters('ontology-'+ontology,'corpora-count','ontology-topic')

    df=pd.read_csv(path_corpora_over_topic_count_ontology,sep=",",encoding="utf-8")
    df.sort_values('corpus',inplace=True,ascending=False)
    ontology_topics=df['ontology-topic'].values
    corpora_count=df['corpus'].values
    if len(corpora_count)>k:
        top_k_corpora_count=corpora_count[:k-1]
        tail_corproa_count= sum(corpora_count[:k-1])
        if tail_corproa_count > corpora_count[0]:
            tail_corproa_count=corpora_count[0]
        top_k_corpora_count=np.append(top_k_corpora_count,[tail_corproa_count])
        top_k_ontology_topics=ontology_topics[:k-1]
        top_k_ontology_topics=np.append(top_k_ontology_topics,['Misc.'])
    else:
        top_k_corpora_count=corpora_count
        top_k_ontology_topics=ontology_topics
        k=len(ontology_topics)

    fig,ax = plt.subplots(figsize=size, subplot_kw=dict(polar=True))
    rlabels=list(range(0,top_k_corpora_count[0],radial_step))
    width = 2*np.pi / k
    theta = np.linspace(0.0, 2 * np.pi, k, endpoint=False)
    colors = plt.cm.viridis(np.linspace(0, 1, k))

    ax.set_rticks(list(rlabels))
    ticks= np.linspace(0,360,k+1)
    ax.set_xticks(np.deg2rad(ticks[:-1]))

    ticklabels =shorten_labels(top_k_ontology_topics,max_label_size)
    ax.set_xticklabels(ticklabels, fontsize=10)


    plt.gcf().canvas.draw()
    angles = np.linspace(0,2*np.pi,len(ax.get_xticklabels())+1)
    angles[np.cos(angles) < 0] = angles[np.cos(angles) < 0] + np.pi
    angles = np.rad2deg(angles)
    labels = []
    for label, angle in zip(ax.get_xticklabels(), angles):
        x,y = label.get_position()
        lab = ax.text(x,y+0.5-offset, label.get_text(), transform=label.get_transform(),
                      ha=label.get_ha(), va=label.get_va())
        lab.set_rotation(angle)
        labels.append(lab)
    ax.set_xticklabels([])

    ax.bar(theta, top_k_corpora_count, width=width, bottom=0.0, color=colors, alpha=0.5)



    fig.savefig(path_figure_corpora_over_topic_ontology_count)


from conf.configuration import *
import pandas as pd
from utils.parts import *
import random
import csv
import numpy as np
from topic_modeling.topics import *
from mylogging.mylogging import *
def produce_threshold_suggestions(dataset,ontology,model,threshold,debug=False):
    path_topic_path_threshold = get_path_topics_per_document_threshold(dataset,ontology,model,threshold)
    topics,topic_ids = load_topics_with_ids('ontology-'+ontology)
    top_document_dict={}
    top_document_dict['id']=[]
    top_document_dict['topic-id']=[]

    for document_vectors_ids in parts_vectors(dataset,ontology,model):
        for document_vector_id in document_vectors_ids:
            document_vector= np.array(document_vector_id[0])
            document_id=document_vector_id[1]
            if isinstance(document_vector,list):
                topics_indexes = np.where(document_vector>=threshold)[0]
                scores = document_vector[topics_indexes]
            else:
                topics_indexes = np.where(document_vector>=threshold)[0]
                scores = document_vector[topics_indexes]

            top_topic_ids = [topic_ids[topic_index] for topic_index in topics_indexes]
            top_document_dict['topic-id'].extend(top_topic_ids)
            top_document_dict['id'].extend([document_id for _ in top_topic_ids])
    df_threshold_topic=pd.DataFrame(top_document_dict)
    log_size("threshold-topics",df_threshold_topic.shape[0])
    df_threshold_topic.to_csv(path_topic_path_threshold,quotechar='"',sep=",",quoting=csv.QUOTE_ALL,encoding="utf-8",index=False)





import argument_esa_model.esa_all_terms
import argument_esa_model.esa_top_n_terms
import tqdm
import numpy as np
from conf.configuration import *
from topic.modtopics import *
import pandas as pd
import sparkpickle
import logging
import os

import codecs
import pickle



def dict_to_np_array(dictionary):
    vector  = []
    for key in sorted(dictionary):
        vector.append(dictionary[key])
    np_arr= np.array(vector)

    return np_arr

def esa_model(topic_ontology,texts):
    path_topic_model = get_path_topic_model('ontology-'+topic_ontology,'esa')
    path_word2vec_model = get_path_topic_model('word2vec','word2vec')
    path_word2vec_vocab = get_path_vocab('word2vec')
    document_vectors = []
    for text in tqdm.tqdm(texts):
        vector = argument_esa_model.esa_all_terms.model_topic(path_topic_model,path_word2vec_model,path_word2vec_vocab,'cos',text)
        document_vectors.append(dict_to_np_array(vector))
    return document_vectors

def word2vec_esa_model(topic_ontology,texts):
    None

def model(topic_ontology,topic_model,texts):
    document_vectors = []
    if topic_model =='esa':
        if topic_ontology=='strategic-intlligence':
            for text in tqdm.tqdm(texts):
                vector = esa_model_strategic_intelligence.process(text, False)
                document_vectors.append(dict_to_np_array(vector))
        if topic_ontology == 'debatepedia':
            for text in tqdm.tqdm(texts):
                vector = esa_model_debatepedia.process(text, False)

                document_vectors.append(dict_to_np_array(vector))
    return document_vectors

def parse_pickle(path):
    with open(path,'rb') as file:
        return sparkpickle.load(file)

def parse_file(path):

    vectors=[]
    ids=[]
    with open(path,'r',encoding='utf-8') as file:
        for i,line in enumerate(file):
            line_without_brackts = line[1:-2]
            tokens = line_without_brackts.split("', '")
            if len(tokens)!=2:
                tokens = line_without_brackts.split("\", '")
                if len(tokens)!=2:
                    #Todo raise error
                    continue
            id = tokens[1][:-1]
            ids.append(id)

            np_array = pickle.loads(codecs.decode(tokens[0][1:].replace('\\n','').encode(), "base64"))

            vectors.append(np_array)
    return vectors,ids


def load_topics(topic_ontology):
    path_topics = get_path_topics(topic_ontology)
    topics_df=pd.read_csv(path_topics).sort_values('name')
    return list(topics_df['name'])

def read_cluster_topic_vectors(dataset,topic_ontology,topic_model):
    granularity = get_granularity(dataset)
    logging.warning("Preprocessing %s with %s and %s"%(dataset,topic_ontology,topic_model))
    if granularity == 'argument':
        path_argument_vectors = get_path_argument_vectors(dataset,topic_ontology,topic_model)
        path_argument_vectors_cluster = get_path_argument_vectors(dataset,topic_ontology,topic_model).replace(".csv","")
        vectors_with_ids= {}
        vectors_with_ids['argument-id']=[]
        vectors_with_ids['argument-vector']=[]
        for root,dirs,files in os.walk(path_argument_vectors_cluster):
            for file in tqdm.tqdm(files):
                if 'part' in file:
                    path = os.path.join(root,file)
                    try:
                        vectors,ids=parse_file(path)

                    except:
                        document_ids=parse_pickle(path)
                        vectors,ids = zip(*document_ids)
                        if len(document_ids)==0:
                            continue


                    vectors_with_ids['argument-id'].extend(ids)
                    vectors_with_ids['argument-vector'].extend(vectors)
        df_argument_vectors=pd.DataFrame(vectors_with_ids)
        if df_argument_vectors.shape[0] > 10000:
            print('yohooo')
            df_argument_vectors.to_parquet(path_argument_vectors)
        else:
            df_argument_vectors.to_pickle(path_argument_vectors)

    elif granularity == 'document':
        path_document_vectors = get_path_document_vectors(dataset,topic_ontology,topic_model)
        path_document_vectors_cluster = get_path_document_vectors(dataset,topic_ontology,topic_model).replace(".csv","")
        logging.warning("Converting vectors form %s to %s"%(path_document_vectors_cluster,path_document_vectors))
        vectors_with_ids= {}
        vectors_with_ids['document-id']=[]
        vectors_with_ids['document-vector']=[]
        part_count=get_part_count(dataset)
        files = os.listdir(path_document_vectors_cluster)
        if part_count!=None:
            logging.warning("found %d parts"%part_count)
            part_size=int(len(files)/part_count)
            for part_num in range(part_count):
                logging.warning("working on %d part"%part_num)
                vectors_with_ids['document-id']=[]
                vectors_with_ids['document-vector']=[]
                if part_num==part_count -1:
                    part_files=files[part_num*part_size:]
                else:
                    part_files=files[part_num*part_size:(part_num+1)*part_size]

                for file in tqdm.tqdm(part_files):
                    if 'part' in file:
                        path = os.path.join(path_document_vectors_cluster,file)
                    logging.warning("Preprocessing %s"%file)
                    document_ids=parse_pickle(path)
                    if len(document_ids)==0:
                        continue
                    vectors,ids = zip(*document_ids)
                    logging.warning(("Found %d vectors"%(len(vectors))))
                    vectors_with_ids['document-id'].extend(ids)
                    vectors_with_ids['document-vector'].extend(vectors)
                logging.warning(("Example id is %s ")%str(vectors_with_ids['document-id'][0]))
                logging.warning(("Example vector is %s ")%str(vectors_with_ids['document-vector'][0]))
                df_document_vectors=pd.DataFrame(vectors_with_ids)
                path_document_vectors_part = get_path_document_vectors_part(dataset,topic_ontology,topic_model,part_num)
                logging.warning(("Saving to part path %s")%path_document_vectors_part)
                df_document_vectors.to_pickle(path_document_vectors_part)
        else:

            for file in tqdm.tqdm(files):
                if 'part' in file:
                    path = os.path.join(path_document_vectors_cluster,file)
                    logging.warning("Preprocessing %s"%file)
                    document_ids=parse_pickle(path)
                    if len(document_ids)==0:
                        continue
                    vectors,ids = zip(*document_ids)
                    logging.warning(("Found %d vectors"%(len(vectors))))
                    vectors_with_ids['document-id'].extend(ids)
                    vectors_with_ids['document-vector'].extend(vectors)
            logging.warning(("Example id is %s ")%str(vectors_with_ids['document-id'][0]))
            logging.warning(("Example vector is %s ")%str(vectors_with_ids['document-vector'][0]))
            df_document_vectors=pd.DataFrame(vectors_with_ids)

            df_document_vectors.to_pickle(path_document_vectors)



def convert_topic_vectors_from_cluster(ontology,model):
    corpora = load_corpora_list()
    logging.warning("preprocessing vectors of %d corpus modeled with %s and %s"%(len(corpora),ontology,model))
    for corpus in corpora:
        logging.warning("preprocessing %s "%corpus)
        read_cluster_topic_vectors(corpus,ontology,model)




def convert_topic_vectors_sample():
    topic_ontologies= get_topic_ontologies()
    models = ['word2vec-esa-100' ]
    for topic_ontology in topic_ontologies:
        logging.warning("reading up %s vectors"%topic_ontology)
        for model in models:
            logging.warning("reading up %s vectors"%model)
            if topic_ontology=='strategic-intelligence-sub-topics':
                read_cluster_topic_vectors('sample-cmv',topic_ontology+"-part1",model)
                read_cluster_topic_vectors('sample-cmv',topic_ontology+"-part2",model)
            read_cluster_topic_vectors('sample-cmv',topic_ontology,model)
    #read_cluster_topic_vectors('sample','debatepedia','word2vec-esa-100')
    #read_cluster_topic_vectors('sample','wikipedia','word2vec-esa-100')
    #read_cluster_topic_vectors('sample','strategic-intelligence','word2vec-esa-100')
    #read_cluster_topic_vectors('sample','wikipedia-categories','word2vec-esa-100')
    #read_cluster_topic_vectors('sample','strategic-intelligence-sub-topics-part1','word2vec-esa-100')
    #read_cluster_topic_vectors('sample','strategic-intelligence-sub-topics-part2','word2vec-esa-100')

    #read_cluster_topic_vectors('sample','debatepedia','word2vec-esa-10')
    #read_cluster_topic_vectors('sample','wikipedia','word2vec-esa-10')
    #read_cluster_topic_vectors('sample','strategic-intelligence','word2vec-esa-10')
    #read_cluster_topic_vectors('sample','wikipedia-categories','word2vec-esa-10')
    #read_cluster_topic_vectors('sample','strategic-intelligence-sub-topics-part1','word2vec-esa-10')
    #read_cluster_topic_vectors('sample','strategic-intelligence-sub-topics-part2','word2vec-esa-10')

def convert_topic_vectors_args_me(topic_ontology,model):
    read_cluster_topic_vectors('args-me',topic_ontology,model)

def convert_topic_vectors_ibm_debater_claim_sentence_search(topic_ontology,model):
    read_cluster_topic_vectors('ibm-debater-claim-sentence-search',topic_ontology,model)

def convert_topic_vectors_kialo(topic_ontology,model):
    read_cluster_topic_vectors('kialo',topic_ontology,model)

def convert_topic_vectors_aifdb(topic_ontology,model):
    read_cluster_topic_vectors('aifdb',topic_ontology,model)

#read_cluster_topic_vectors('test-sample','debatepedia','word2vec-esa-1')
#read_cluster_topic_vectors('test-sample','wikipedia','word2vec-esa-1')
#read_cluster_topic_vectors('test-sample','wikipedia-categories','word2vec-esa-1')
#read_cluster_topic_vectors('test-sample','strategic-intelligence','word2vec-esa-1')


#convert_topic_vectors_aifdb('debatepedia','esa')
## convert_topic_vectors_kialo('debatepedia','esa')
#convert_topic_vectors_ibm_debater_claim_sentence_search('debatepedia','esa')
#convert_topic_vectors_ibm_debater_claim_sentence_search('wikipedia','esa')
#convert_topic_vectors_ibm_debater_claim_sentence_search('strategic-intelligence','esa')
#convert_topic_vectors_args_me('debatepedia','esa')

#convert_topic_vectors_aifdb('strategic-intelligence','esa')
#convert_topic_vectors_kialo('strategic-intelligence','esa')
#convert_topic_vectors_args_me('strategic-intelligence','esa')
#convert_topic_vectors_args_me('strategic-intelligence','esa')

#convert_topic_vectors_aifdb('wikipedia','esa')
#convert_topic_vectors_kialo('wikipedia','esa')
#convert_topic_vectors_args_me('wikipedia','esa')


#convert_topic_vectors_sample()
#convert_topic_vectors_args_me('debatepedia','esa')